<?php
/**
 * Based on the Genesis Featured Post widget (StudioPress) and Winning Agent Pro Communities widget (Carrie Dils).
 *
 * @package Agent Focused Pro / Widgets
 * @author  Marcy Diaz for Winning Agent
 * @subpackage Customizations
 * @license GPL-2.0+
 */

/**
 * Agent Focused Pro Featured Community widget class.
 *
 * @since 1.0.0
 */
class Agent_Focused_Pro_Featured_Community extends WP_Widget {

	/**
	 * Holds widget settings defaults, populated in constructor.
	 *
	 * @var array
	 */
	protected $defaults;

	/**
	 * Constructor. Set the default widget options and create widget.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		$this->defaults = [
			'title'         => '',
			'posts_num'     => 1,
			'posts_offset'  => 0,
			'orderby'       => '',
			'order'         => '',
			'show_image'    => 0,
			'image_size'    => '',
			'show_title'    => 0,
			'show_content'  => 'excerpt',
			'content_limit' => '',
			'more_text'     => __( '[Read More...]', 'agentfocused' ),

		];

		$widget_ops = [
			'classname'   => 'featured-content featured-community',
			'description' => __( 'Displays featured communities with thumbnails', 'agentfocused' ),
		];

		$control_ops = [
			'id_base' => 'featured-community',
			'width'   => 505,
			'height'  => 350,
		];

		parent::__construct( 'featured-community', __( 'Featured Communities', 'agentfocused' ), $widget_ops, $control_ops );

	}

	/**
	 * Echo the widget content.
	 *
	 * @since 1.0.0
	 *
	 * @global WP_Query $wp_query               Query object.
	 * @global array    $_genesis_displayed_ids Array of displayed post IDs.
	 * @global int      $more
	 *
	 * @param array $args     Display arguments including `before_title`, `after_title`,
	 *                        `before_widget`, and `after_widget`.
	 * @param array $instance The settings for the particular instance of the widget.
	 */
	public function widget( $args, $instance ) {

		global $wp_query, $_genesis_displayed_ids;

		// Merge with defaults.
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $args['before_widget'];

		// Set up the widget title.
		if ( ! empty( $instance['title'] ) ) {
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'], $instance, $this->id_base ) . $args['after_title'];
		}

		$query_args = [
			'post_type' => 'wap-community',
			'showposts' => $instance['posts_num'],
			'offset'    => $instance['posts_offset'],
			'orderby'   => $instance['orderby'],
			'order'     => $instance['order'],
		];

		$wp_query = new WP_Query( $query_args ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.OverrideProhibited -- Reset later.

		if ( have_posts() ) {

			while ( have_posts() ) {

				the_post();

				$_genesis_displayed_ids[] = get_the_ID();

				genesis_markup(
					[
						'open'    => '<article %s>',
						'context' => 'entry',
						'params'  => [
							'is_widget' => true,
						],
					]
				);

				$image = genesis_get_image(
					[
						'format'  => 'html',
						'size'    => $instance['image_size'],
						'context' => 'featured-community-widget',
						'attr'    => genesis_parse_attr( 'entry-image-widget', [] ),
					]
				);

				if ( $image && $instance['show_image'] ) {

					$image_alignment = 'alignnone';
					$role            = empty( $instance['show_title'] ) ? '' : ' aria-hidden="true" tabindex="-1"';

					printf(
						'<a href="%s" class="%s"%s>%s</a>',
						esc_url( get_permalink() ),
						esc_attr( $image_alignment ),
						$role, // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- Escaping breaks output here
						wp_make_content_images_responsive( $image )
					);
				}

				if ( $instance['show_title'] ) {

					$header = '';

					if ( ! empty( $instance['show_title'] ) ) {

						$title = get_the_title() ?: __( '(no title)', 'agentfocused' );

/**
						 * Filter the featured post widget title.
						 *
						 * @since 2.2.0
						 *
						 * @param string $title    Featured post title.
						 * @param array  $instance {
						 *     Widget settings for this instance.
						 *
						 *     @type string $title                   Widget title.
						 *     @type int    $posts_cat               ID of the post category.
						 *     @type int    $posts_num               Number of posts to show.
						 *     @type int    $posts_offset            Number of posts to skip when
						 *                                           retrieving.
						 *     @type string $orderby                 Field to order posts by.
						 *     @type string $order                   ASC for ascending order, DESC for
						 *                                           descending order of posts.
						 *     @type bool   $show_image              True if featured image should be
						 *                                           shown, false otherwise.
						 *     @type string $image_size              Name of the image size.
						 *     @type bool   $show_title              True if featured page title should
						 *                                           be shown, false otherwise.
						 *     @type bool   $show_content            True if featured page content
						 *                                           should be shown, false otherwise.
						 *     @type int    $content_limit           Amount of content to show, in
						 *                                           characters.
						 *     @type int    $more_text               Text to use for More link.
						 * }
						 * @param array  $args     {
						 *     Widget display arguments.
						 *
						 *     @type string $before_widget Markup or content to display before the widget.
						 *     @type string $before_title  Markup or content to display before the widget title.
						 *     @type string $after_title   Markup or content to display after the widget title.
						 *     @type string $after_widget  Markup or content to display after the widget.
						 * }
						 */
						$title   = apply_filters( 'genesis_featured_post_title', $title, $instance, $args );
						$heading = genesis_a11y( 'headings' ) ? 'h4' : 'h2';

						$header .= genesis_markup(
							[
								'open'    => "<{$heading} %s>",
								'close'   => "</{$heading}>",
								'context' => 'entry-title',
								'content' => sprintf( '<a href="%s">%s</a>', get_permalink(), $title ),
								'params'  => [
									'is_widget' => true,
									'wrap'      => $heading,
								],
								'echo'    => false,
							]
						);
					}

					genesis_markup(
						[
							'open'    => '<header %s>',
							'close'   => '</header>',
							'context' => 'entry-header',
							'params'  => [
								'is_widget' => true,
							],
							'content' => $header,
						]
					);
				}

				if ( ! empty( $instance['show_content'] ) ) {

					genesis_markup(
						[
							'open'    => '<div %s>',
							'context' => 'entry-content',
							'params'  => [
								'is_widget' => true,
							],
						]
					);

					if ( 'excerpt' === $instance['show_content'] ) {
						the_excerpt();
					} elseif ( 'content-limit' === $instance['show_content'] ) {
						the_content_limit( (int) $instance['content_limit'], genesis_a11y_more_link( esc_html( $instance['more_text'] ) ) );
					} else {
						global $more;

						$orig_more = $more;
						$more      = 0; // phpcs:ignore WordPress.WP.GlobalVariablesOverride.OverrideProhibited -- Temporary change.

						the_content( genesis_a11y_more_link( esc_html( $instance['more_text'] ) ) );

						$more = $orig_more; // phpcs:ignore WordPress.WP.GlobalVariablesOverride.OverrideProhibited -- Global is being restored.
					}

					genesis_markup(
						[
							'close'   => '</div>',
							'context' => 'entry-content',
							'params'  => [
								'is_widget' => true,
							],
						]
					);
				}

				genesis_markup(
					[
						'close'   => '</article>',
						'context' => 'entry',
						'params'  => [
							'is_widget' => true,
						],
					]
				);

			}
		}

		// Restore original query.
		wp_reset_query(); // phpcs:ignore WordPress.WP.DiscouragedFunctions.wp_reset_query_wp_reset_query -- Making sure the query is really reset.

		echo $args['after_widget'];

	}

	/**
	 * Update a particular instance.
	 *
	 * This function should check that $new_instance is set correctly.
	 * The newly calculated value of $instance should be returned.
	 * If "false" is returned, the instance won't be saved/updated.
	 *
	 * @since 1.0.0
	 *
	 * @param array $new_instance New settings for this instance as input by the user via `form()`.
	 * @param array $old_instance Old settings for this instance.
	 * @return array Settings to save or bool false to cancel saving.
	 */
	public function update( $new_instance, $old_instance ) {

		$post_num = (int) $new_instance['posts_num'];

		$new_instance['posts_num'] = $post_num > 0 && $post_num < 100 ? $post_num : 1;
		$new_instance['title']     = wp_strip_all_tags( $new_instance['title'] );
		$new_instance['more_text'] = wp_strip_all_tags( $new_instance['more_text'] );

		return $new_instance;
	}

	/**
	 * Echo the settings update form.
	 *
	 * @since 1.0.0
	 *
	 * @param array $instance Current settings.
	 * @return void
	 */
	public function form( $instance ) {

		// Merge with defaults.
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		?>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title', 'agentfocused' ); ?>:</label>
			<input type="text" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo esc_attr( $instance['title'] ); ?>" class="widefat" />
		</p>
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'posts_num' ) ); ?>"><?php esc_html_e( 'Number of Communities to Show', 'agentfocused' ); ?>:</label>
			<input type="text" id="<?php echo esc_attr( $this->get_field_id( 'posts_num' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'posts_num' ) ); ?>" value="<?php echo esc_attr( $instance['posts_num'] ); ?>" size="2" placeholder="1" />
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'posts_offset' ) ); ?>"><?php esc_html_e( 'Number of Posts to Offset', 'agentfocused' ); ?>:</label>
			<input type="text" id="<?php echo esc_attr( $this->get_field_id( 'posts_offset' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'posts_offset' ) ); ?>" value="<?php echo esc_attr( $instance['posts_offset'] ); ?>" size="2" />
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'orderby' ) ); ?>"><?php esc_html_e( 'Order By', 'agentfocused' ); ?>:</label>
			<select id="<?php echo esc_attr( $this->get_field_id( 'orderby' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'orderby' ) ); ?>">
				<option value="date" <?php selected( 'date', $instance['orderby'] ); ?>><?php esc_html_e( 'Date Published', 'agentfocused' ); ?></option>
				<option value="title" <?php selected( 'title', $instance['orderby'] ); ?>><?php esc_html_e( 'Title', 'agentfocused' ); ?></option>
				<option value="parent" <?php selected( 'parent', $instance['orderby'] ); ?>><?php esc_html_e( 'Parent', 'agentfocused' ); ?></option>
				<option value="ID" <?php selected( 'ID', $instance['orderby'] ); ?>><?php esc_html_e( 'ID', 'agentfocused' ); ?></option>
				<option value="rand" <?php selected( 'rand', $instance['orderby'] ); ?>><?php esc_html_e( 'Random', 'agentfocused' ); ?></option>
			</select>
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'order' ) ); ?>"><?php esc_html_e( 'Sort Order', 'agentfocused' ); ?>:</label>
			<select id="<?php echo esc_attr( $this->get_field_id( 'order' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'order' ) ); ?>">
				<option value="DESC" <?php selected( 'DESC', $instance['order'] ); ?>><?php esc_html_e( 'Descending (3, 2, 1)', 'agentfocused' ); ?></option>
				<option value="ASC" <?php selected( 'ASC', $instance['order'] ); ?>><?php esc_html_e( 'Ascending (1, 2, 3)', 'agentfocused' ); ?></option>
			</select>
		</p>

		<hr class="div" />

		<p>
			<input id="<?php echo esc_attr( $this->get_field_id( 'show_image' ) ); ?>" type="checkbox" name="<?php echo esc_attr( $this->get_field_name( 'show_image' ) ); ?>" value="1" <?php checked( $instance['show_image'] ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_image' ) ); ?>"><?php esc_html_e( 'Show Featured Image', 'agentfocused' ); ?></label>
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'image_size' ) ); ?>"><?php esc_html_e( 'Image Size', 'agentfocused' ); ?>:</label>
			<select id="<?php echo esc_attr( $this->get_field_id( 'image_size' ) ); ?>" class="genesis-image-size-selector" name="<?php echo esc_attr( $this->get_field_name( 'image_size' ) ); ?>">
				<?php
				$sizes = genesis_get_image_sizes();
				foreach ( $sizes as $name => $size ) {
					printf( '<option value="%s" %s>%s (%sx%s)</option>', esc_attr( $name ), selected( $name, $instance['image_size'], false ), esc_html( $name ), esc_html( $size['width'] ), esc_html( $size['height'] ) );
				}
				?>
			</select>
		</p>

		<hr class="div" />

		<p>
			<input id="<?php echo esc_attr( $this->get_field_id( 'show_title' ) ); ?>" type="checkbox" name="<?php echo esc_attr( $this->get_field_name( 'show_title' ) ); ?>" value="1" <?php checked( $instance['show_title'] ); ?>/>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_title' ) ); ?>"><?php esc_html_e( 'Show Community Title', 'agentfocused' ); ?></label>
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_content' ) ); ?>"><?php esc_html_e( 'Content Type', 'agentfocused' ); ?>:</label>
			<select id="<?php echo esc_attr( $this->get_field_id( 'show_content' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_content' ) ); ?>">
				<option value="content" <?php selected( 'content', $instance['show_content'] ); ?>><?php esc_html_e( 'Show Content', 'agentfocused' ); ?></option>
				<option value="excerpt" <?php selected( 'excerpt', $instance['show_content'] ); ?>><?php esc_html_e( 'Show Excerpt', 'agentfocused' ); ?></option>
				<option value="content-limit" <?php selected( 'content-limit', $instance['show_content'] ); ?>><?php esc_html_e( 'Show Content Limit', 'agentfocused' ); ?></option>
				<option value="" <?php selected( '', $instance['show_content'] ); ?>><?php esc_html_e( 'No Content', 'agentfocused' ); ?></option>
			</select>
			<br />
			<label for="<?php echo esc_attr( $this->get_field_id( 'content_limit' ) ); ?>"><?php esc_html_e( 'Limit content to', 'agentfocused' ); ?>
				<input type="text" id="<?php echo esc_attr( $this->get_field_id( 'content_limit' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'content_limit' ) ); ?>" value="<?php echo esc_attr( (int) $instance['content_limit'] ); ?>" size="3" />
				<?php esc_html_e( 'characters', 'agentfocused' ); ?>
			</label>
		</p>

		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'more_text' ) ); ?>"><?php esc_html_e( 'More Text (if applicable)', 'agentfocused' ); ?>:</label>
			<input type="text" id="<?php echo esc_attr( $this->get_field_id( 'more_text' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'more_text' ) ); ?>" value="<?php echo esc_attr( $instance['more_text'] ); ?>" />
		</p>

		<?php
	}
}
